/* 📖 Bindery Controls v1.0.0 */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.BinderyControls = factory());
}(this, (function () { 'use strict';

function ___$insertStyle(css) {
  if (!css) {
    return;
  }
  if (typeof window === 'undefined') {
    return;
  }

  var style = document.createElement('style');

  style.setAttribute('type', 'text/css');
  style.innerHTML = css;
  document.head.appendChild(style);

  return css;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

/*!
 * Cross-Browser Split 1.1.1
 * Copyright 2007-2012 Steven Levithan <stevenlevithan.com>
 * Available under the MIT License
 * ECMAScript compliant, uniform cross-browser split method
 */

/**
 * Splits a string into an array of strings using a regex or string separator. Matches of the
 * separator are not included in the result array. However, if `separator` is a regex that contains
 * capturing groups, backreferences are spliced into the result each time `separator` is matched.
 * Fixes browser bugs compared to the native `String.prototype.split` and can be used reliably
 * cross-browser.
 * @param {String} str String to split.
 * @param {RegExp|String} separator Regex or string to use for separating the string.
 * @param {Number} [limit] Maximum number of items to include in the result array.
 * @returns {Array} Array of substrings.
 * @example
 *
 * // Basic use
 * split('a b c d', ' ');
 * // -> ['a', 'b', 'c', 'd']
 *
 * // With limit
 * split('a b c d', ' ', 2);
 * // -> ['a', 'b']
 *
 * // Backreferences in result array
 * split('..word1 word2..', /([a-z]+)(\d+)/i);
 * // -> ['..', 'word', '1', ' ', 'word', '2', '..']
 */
var browserSplit = (function split(undef) {

  var nativeSplit = String.prototype.split,
    compliantExecNpcg = /()??/.exec("")[1] === undef,
    // NPCG: nonparticipating capturing group
    self;

  self = function(str, separator, limit) {
    // If `separator` is not a regex, use `nativeSplit`
    if (Object.prototype.toString.call(separator) !== "[object RegExp]") {
      return nativeSplit.call(str, separator, limit);
    }
    var output = [],
      flags = (separator.ignoreCase ? "i" : "") + (separator.multiline ? "m" : "") + (separator.extended ? "x" : "") + // Proposed for ES6
      (separator.sticky ? "y" : ""),
      // Firefox 3+
      lastLastIndex = 0,
      // Make `global` and avoid `lastIndex` issues by working with a copy
      separator = new RegExp(separator.source, flags + "g"),
      separator2, match, lastIndex, lastLength;
    str += ""; // Type-convert
    if (!compliantExecNpcg) {
      // Doesn't need flags gy, but they don't hurt
      separator2 = new RegExp("^" + separator.source + "$(?!\\s)", flags);
    }
    /* Values for `limit`, per the spec:
     * If undefined: 4294967295 // Math.pow(2, 32) - 1
     * If 0, Infinity, or NaN: 0
     * If positive number: limit = Math.floor(limit); if (limit > 4294967295) limit -= 4294967296;
     * If negative number: 4294967296 - Math.floor(Math.abs(limit))
     * If other: Type-convert, then use the above rules
     */
    limit = limit === undef ? -1 >>> 0 : // Math.pow(2, 32) - 1
    limit >>> 0; // ToUint32(limit)
    while (match = separator.exec(str)) {
      // `separator.lastIndex` is not reliable cross-browser
      lastIndex = match.index + match[0].length;
      if (lastIndex > lastLastIndex) {
        output.push(str.slice(lastLastIndex, match.index));
        // Fix browsers whose `exec` methods don't consistently return `undefined` for
        // nonparticipating capturing groups
        if (!compliantExecNpcg && match.length > 1) {
          match[0].replace(separator2, function() {
            for (var i = 1; i < arguments.length - 2; i++) {
              if (arguments[i] === undef) {
                match[i] = undef;
              }
            }
          });
        }
        if (match.length > 1 && match.index < str.length) {
          Array.prototype.push.apply(output, match.slice(1));
        }
        lastLength = match[0].length;
        lastLastIndex = lastIndex;
        if (output.length >= limit) {
          break;
        }
      }
      if (separator.lastIndex === match.index) {
        separator.lastIndex++; // Avoid an infinite loop
      }
    }
    if (lastLastIndex === str.length) {
      if (lastLength || !separator.test("")) {
        output.push("");
      }
    } else {
      output.push(str.slice(lastLastIndex));
    }
    return output.length > limit ? output.slice(0, limit) : output;
  };

  return self;
})();

var indexOf = [].indexOf;

var indexof = function(arr, obj){
  if (indexOf) return arr.indexOf(obj);
  for (var i = 0; i < arr.length; ++i) {
    if (arr[i] === obj) return i;
  }
  return -1;
};

// contains, add, remove, toggle


var classList = ClassList;

function ClassList(elem) {
    var cl = elem.classList;

    if (cl) {
        return cl
    }

    var classList = {
        add: add
        , remove: remove
        , contains: contains
        , toggle: toggle
        , toString: $toString
        , length: 0
        , item: item
    };

    return classList

    function add(token) {
        var list = getTokens();
        if (indexof(list, token) > -1) {
            return
        }
        list.push(token);
        setTokens(list);
    }

    function remove(token) {
        var list = getTokens()
            , index = indexof(list, token);

        if (index === -1) {
            return
        }

        list.splice(index, 1);
        setTokens(list);
    }

    function contains(token) {
        return indexof(getTokens(), token) > -1
    }

    function toggle(token) {
        if (contains(token)) {
            remove(token);
            return false
        } else {
            add(token);
            return true
        }
    }

    function $toString() {
        return elem.className
    }

    function item(index) {
        var tokens = getTokens();
        return tokens[index] || null
    }

    function getTokens() {
        var className = elem.className;

        return filter(className.split(" "), isTruthy)
    }

    function setTokens(list) {
        var length = list.length;

        elem.className = list.join(" ");
        classList.length = length;

        for (var i = 0; i < list.length; i++) {
            classList[i] = list[i];
        }

        delete list[length];
    }
}

function filter (arr, fn) {
    var ret = [];
    for (var i = 0; i < arr.length; i++) {
        if (fn(arr[i])) ret.push(arr[i]);
    }
    return ret
}

function isTruthy(value) {
    return !!value
}

/**
 * Utils for HTML attributes
 * @module html-attributes
 */

// property to attribute names
var PROPS_TO_ATTRS = {
  'className': 'class',
  'htmlFor': 'for',
};

// map of attributes to the elements they affect
// see https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes
var HTML_ATTRIBUTES = {
  'accept': new Set([
    'form',
    'input',
  ]),

  'accept-charset': new Set([
    'form',
  ]),

  'accesskey': 'GLOBAL',

  'action': new Set([
    'form',
  ]),

  'align': new Set([
    'applet',
    'caption',
    'col',
    'colgroup',
    'hr',
    'iframe',
    'img',
    'table',
    'tbody',
    'td',
    'tfoot',
    'th',
    'thead',
    'tr',
  ]),

  'alt': new Set([
    'applet',
    'area',
    'img',
    'input',
  ]),

  'async': new Set([
    'script',
  ]),

  'autocomplete': new Set([
    'form',
    'input',
  ]),

  'autofocus': new Set([
    'button',
    'input',
    'keygen',
    'select',
    'textarea',
  ]),

  'autoplay': new Set([
    'audio',
    'video',
  ]),

  'autosave': new Set([
    'input',
  ]),

  'bgcolor': new Set([
    'body',
    'col',
    'colgroup',
    'marquee',
    'table',
    'tbody',
    'tfoot',
    'td',
    'th',
    'tr',
  ]),

  'border': new Set([
    'img',
    'object',
    'table',
  ]),

  'buffered': new Set([
    'audio',
    'video',
  ]),

  'challenge': new Set([
    'keygen',
  ]),

  'charset': new Set([
    'meta',
    'script',
  ]),

  'checked': new Set([
    'command',
    'input',
  ]),

  'cite': new Set([
    'blockquote',
    'del',
    'ins',
    'q',
  ]),

  'class': 'GLOBAL',

  'code': new Set([
    'applet',
  ]),

  'codebase': new Set([
    'applet',
  ]),

  'color': new Set([
    'basefont',
    'font',
    'hr',
  ]),

  'cols': new Set([
    'textarea',
  ]),

  'colspan': new Set([
    'td',
    'th',
  ]),

  'content': new Set([
    'meta',
  ]),

  'contenteditable': 'GLOBAL',

  'contextmenu': 'GLOBAL',

  'controls': new Set([
    'audio',
    'video',
  ]),

  'coords': new Set([
    'area',
  ]),

  'data': new Set([
    'object',
  ]),

  'datetime': new Set([
    'del',
    'ins',
    'time',
  ]),

  'default': new Set([
    'track',
  ]),

  'defer': new Set([
    'script',
  ]),

  'dir': 'GLOBAL',

  'dirname': new Set([
    'input',
    'textarea',
  ]),

  'disabled': new Set([
    'button',
    'command',
    'fieldset',
    'input',
    'keygen',
    'optgroup',
    'option',
    'select',
    'textarea',
  ]),

  'download': new Set([
    'a',
    'area',
  ]),

  'draggable': 'GLOBAL',

  'dropzone': 'GLOBAL',

  'enctype': new Set([
    'form',
  ]),

  'for': new Set([
    'label',
    'output',
  ]),

  'form': new Set([
    'button',
    'fieldset',
    'input',
    'keygen',
    'label',
    'meter',
    'object',
    'output',
    'progress',
    'select',
    'textarea',
  ]),

  'formaction': new Set([
    'input',
    'button',
  ]),

  'headers': new Set([
    'td',
    'th',
  ]),

  'height': new Set([
    'canvas',
    'embed',
    'iframe',
    'img',
    'input',
    'object',
    'video',
  ]),

  'hidden': 'GLOBAL',

  'high': new Set([
    'meter',
  ]),

  'href': new Set([
    'a',
    'area',
    'base',
    'link',
  ]),

  'hreflang': new Set([
    'a',
    'area',
    'link',
  ]),

  'http-equiv': new Set([
    'meta',
  ]),

  'icon': new Set([
    'command',
  ]),

  'id': 'GLOBAL',

  'ismap': new Set([
    'img',
  ]),

  'itemprop': 'GLOBAL',

  'keytype': new Set([
    'keygen',
  ]),

  'kind': new Set([
    'track',
  ]),

  'label': new Set([
    'track',
  ]),

  'lang': 'GLOBAL',

  'language': new Set([
    'script',
  ]),

  'list': new Set([
    'input',
  ]),

  'loop': new Set([
    'audio',
    'bgsound',
    'marquee',
    'video',
  ]),

  'low': new Set([
    'meter',
  ]),

  'manifest': new Set([
    'html',
  ]),

  'max': new Set([
    'input',
    'meter',
    'progress',
  ]),

  'maxlength': new Set([
    'input',
    'textarea',
  ]),

  'media': new Set([
    'a',
    'area',
    'link',
    'source',
    'style',
  ]),

  'method': new Set([
    'form',
  ]),

  'min': new Set([
    'input',
    'meter',
  ]),

  'multiple': new Set([
    'input',
    'select',
  ]),

  'muted': new Set([
    'video',
  ]),

  'name': new Set([
    'button',
    'form',
    'fieldset',
    'iframe',
    'input',
    'keygen',
    'object',
    'output',
    'select',
    'textarea',
    'map',
    'meta',
    'param',
  ]),

  'novalidate': new Set([
    'form',
  ]),

  'open': new Set([
    'details',
  ]),

  'optimum': new Set([
    'meter',
  ]),

  'pattern': new Set([
    'input',
  ]),

  'ping': new Set([
    'a',
    'area',
  ]),

  'placeholder': new Set([
    'input',
    'textarea',
  ]),

  'poster': new Set([
    'video',
  ]),

  'preload': new Set([
    'audio',
    'video',
  ]),

  'radiogroup': new Set([
    'command',
  ]),

  'readonly': new Set([
    'input',
    'textarea',
  ]),

  'rel': new Set([
    'a',
    'area',
    'link',
  ]),

  'required': new Set([
    'input',
    'select',
    'textarea',
  ]),

  'reversed': new Set([
    'ol',
  ]),

  'rows': new Set([
    'textarea',
  ]),

  'rowspan': new Set([
    'td',
    'th',
  ]),

  'sandbox': new Set([
    'iframe',
  ]),

  'scope': new Set([
    'th',
  ]),

  'scoped': new Set([
    'style',
  ]),

  'seamless': new Set([
    'iframe',
  ]),

  'selected': new Set([
    'option',
  ]),

  'shape': new Set([
    'a',
    'area',
  ]),

  'size': new Set([
    'input',
    'select',
  ]),

  'sizes': new Set([
    'img',
    'link',
    'source',
  ]),

  'span': new Set([
    'col',
    'colgroup',
  ]),

  'spellcheck': 'GLOBAL',

  'src': new Set([
    'audio',
    'embed',
    'iframe',
    'img',
    'input',
    'script',
    'source',
    'track',
    'video',
  ]),

  'srcdoc': new Set([
    'iframe',
  ]),

  'srclang': new Set([
    'track',
  ]),

  'srcset': new Set([
    'img',
  ]),

  'start': new Set([
    'ol',
  ]),

  'step': new Set([
    'input',
  ]),

  'style': 'GLOBAL',

  'summary': new Set([
    'table',
  ]),

  'tabindex': 'GLOBAL',

  'target': new Set([
    'a',
    'area',
    'base',
    'form',
  ]),

  'title': 'GLOBAL',

  'type': new Set([
    'button',
    'input',
    'command',
    'embed',
    'object',
    'script',
    'source',
    'style',
    'menu',
  ]),

  'usemap': new Set([
    'img',
    'input',
    'object',
  ]),

  'value': new Set([
    'button',
    'option',
    'input',
    'li',
    'meter',
    'progress',
    'param',
  ]),

  'width': new Set([
    'canvas',
    'embed',
    'iframe',
    'img',
    'input',
    'object',
    'video',
  ]),

  'wrap': new Set([
    'textarea',
  ]),
};

function isStandardAttribute(attrName, tagName) {
  tagName = tagName.toLowerCase();
  var attr = HTML_ATTRIBUTES[attrName.toLowerCase()];
  return !!attr && (
    attr === 'GLOBAL' ||
    attr.has(tagName)
  );
}

function propToAttr(prop) {
  return PROPS_TO_ATTRS[prop] || prop;
}

var htmlAttributes = {
  isStandardAttribute: isStandardAttribute,
  propToAttr: propToAttr,
};

function Document() {}

Document.prototype.createTextNode = function(v) {
    var n = new Text();
    n.textContent = v;
    n.nodeName = '#text';
    n.nodeType = 3;
    return n;
};

Document.prototype.createElement = function(nodeName) {
    var el = new Element();
    el.nodeName = el.tagName = nodeName;
    return el;
};

Document.prototype.createComment = function(data) {
    var el = new Comment();
    el.data = data;
    return el;
};


function Node () {}

Text.prototype = new Node();

Element.prototype = new Node();

Comment.prototype = new Node();


function Style (el) {
  this.el = el;
  this.styles = [];
}

Style.prototype.setProperty = function (n,v) {
    this.el._setProperty(this.styles, {name: n, value:v});
};

Style.prototype.getProperty = function(n) {
    return this.el._getProperty(this.styles, n);
};

Style.prototype.__defineGetter__('cssText', function () {
    var stylified = '';
    this.styles.forEach(function(s){
      stylified+=s.name+':'+s.value+';';
    });
    return stylified;
});

Style.prototype.__defineSetter__('cssText', function (v) {
    this.styles.length = 0;

    // parse cssText and set style attributes
    v.split(';').forEach(function(part){
      var splitPoint = part.indexOf(':');
      if (splitPoint){
        var key = part.slice(0, splitPoint).trim();
        var value = part.slice(splitPoint+1).trim();
        this.setProperty(key, value);
      }
    }, this);
});

function Attribute(name, value){
  if (name) {
    this.name = name;
    this.value = value ? value : '';
  }
}


function Element() {
    var self = this;

    this.style = new Style(this);
    this.classList = classList(this);
    this.childNodes = [];
    this.attributes = [];
    this.dataset = {};
    this.className = '';

    this._setProperty = function(arr, obj, key, val) {
      var p = self._getProperty(arr, key);
      if (p) {
        p.value = String(val);
        return;
      }
      arr.push('function' === typeof obj ? new obj(key.toLowerCase(),String(val)) : obj);
    };

    this._getProperty = function (arr, key) {
      if (!key) return;
      key = key.toLowerCase();
      for (var i=0;i<arr.length;i++) {
        if (key == arr[i].name) return arr[i];
      }
    };
}

Element.prototype.nodeType = 1;

Element.prototype.appendChild = function (child) {
    child.parentElement = this;
    this.childNodes.push(child);
    return child;
};

Element.prototype.setAttribute = function (n, v) {
  if (n == 'style'){
    this.style.cssText = v;
  } else {
    this._setProperty(this.attributes, Attribute, n, v);
  }
};

Element.prototype.getAttribute = function (n) {
  if (n == 'style'){
    return this.style.cssText
  } else {
    var result = this._getProperty(this.attributes, n);
    return typeof result !== 'undefined' ? result.value : null;
  }
};

Element.prototype.removeAttribute = function (n) {
  if (n === 'class') {
    delete this.className;
  } else {
    for (var i = 0, len = this.attributes.length; i < len; i++) {
      if (this.attributes[i].name === n) {
        this.attributes.splice(i, 1);
        break;
      }
    }
  }
};

Element.prototype.replaceChild = function(newChild, oldChild) {
    var self = this;
    var replaced = false;
    this.childNodes.forEach(function(child, index){
        if (child === oldChild) {
            self.childNodes[index] = newChild;
            newChild.parentElement = this;
            replaced = true;
        }
    });
    if (replaced) return oldChild;
};

Element.prototype.removeChild = function(rChild) {
    var self = this;
    var removed = true;
    this.childNodes.forEach(function(child, index){
        if (child === rChild) {
          // use splice to keep a clean childNode array
          self.childNodes.splice(index, 1);
          rChild.parentElement = null;
          removed = true;
        }
    });
    if (removed) return rChild;
};

Element.prototype.insertBefore = function (newChild, existingChild) {
  var childNodes = this.childNodes;

  if (existingChild === null) {
    childNodes.push(newChild);
  } else {
    for (var i = 0, len = childNodes.length; i < len; i++) {
      var child = childNodes[i];
      if (child === existingChild) {
        i === 0 ? childNodes.unshift(newChild) : childNodes.splice(i, 0, newChild);
        break;
      }
    }
  }
  newChild.parentElement = this;

  return newChild;
};

Element.prototype.addEventListener = function(type, listener, useCapture, wantsUntrusted) {
  // https://developer.mozilla.org/en-US/docs/Web/API/EventTarget.addEventListener
  // There is an implementation there but probably not worth it.
};

Element.prototype.removeEventListener = function(type, listener, useCapture) {
  // https://developer.mozilla.org/en/docs/Web/API/EventTarget.removeEventListener
  // There is an implementation there but probably not worth it.
};

Element.prototype.insertAdjacentHTML = function(position, text) {
  // https://developer.mozilla.org/en-US/docs/Web/API/Element.insertAdjacentHTML
  // Not too much work to implement similar to innerHTML below.
};

Element.prototype.__defineGetter__('innerHTML', function () {
    // regurgitate set innerHTML
    var s = this.childNodes.html || '';
    this.childNodes.forEach(function (e) {
      s += (e.outerHTML || e.textContent);
    });
    return s
});

Element.prototype.__defineSetter__('innerHTML', function (v) {
    //only handle this simple case that doesn't need parsing
    //this case is useful... parsing is hard and will need added deps!
    this.childNodes.length = 0;

    // hack to preserve set innerHTML - no parsing just regurgitation
    this.childNodes.html = v;
});


Element.prototype.__defineGetter__('outerHTML', function () {
  var a = [],  self = this;
  var VOID_ELEMENTS = {
    AREA: true,
    BASE: true,
    BR: true,
    COL: true,
    EMBED: true,
    HR: true,
    IMG: true,
    INPUT: true,
    KEYGEN: true,
    LINK: true,
    META: true,
    PARAM: true,
    SOURCE: true,
    TRACK: true,
    WBR: true
  };

  function _stringify(arr) {
    var attr = [], value;
    arr.forEach(function(a){
      value = ('style' != a.name) ? a.value : self.style.cssText;
      attr.push(a.name+'='+'\"'+escapeAttribute(value)+'\"');
    });
    return attr.length ? ' '+attr.join(" ") : '';
  }

  function _dataify(data) {
    var attr = [], value;
    Object.keys(data).forEach(function(name){
      attr.push('data-'+name+'='+'\"'+escapeAttribute(data[name])+'\"');
    });
    return attr.length ? ' '+attr.join(" ") : '';
  }

  function _propertify() {
    var props = [];
    for (var key in self) {
      var attrName = htmlAttributes.propToAttr(key);
      if (
        self.hasOwnProperty(key) &&
        ['string', 'boolean', 'number'].indexOf(typeof self[key]) !== -1 &&
        htmlAttributes.isStandardAttribute(attrName, self.nodeName) &&
        _shouldOutputProp(key, attrName)
      ) {
        props.push({name: attrName, value: self[key]});
      }
    }
    return props ? _stringify(props) : '';
  }

  function _shouldOutputProp(prop, attr) {
    if (self.getAttribute(attr)) {
      // let explicitly-set attributes override props
      return false;
    } else {
      if (prop === 'className' && !self[prop]) {
        return false;
      }
    }
    return true;
  }

  var attrs = this.style.cssText ? this.attributes.concat([{name: 'style'}]) : this.attributes;

  a.push('<'+this.nodeName + _propertify() + _stringify(attrs) + _dataify(this.dataset) +'>');

  if (!VOID_ELEMENTS[this.nodeName.toUpperCase()]){
    a.push(this.innerHTML);
    a.push('</'+this.nodeName+'>');
  }

  return a.join('')
});

Element.prototype.__defineGetter__('textContent', function () {
  var s = '';
  this.childNodes.forEach(function (e) {
    s += e.textContent;
  });
  return s
});

Element.prototype.__defineSetter__('textContent', function (v) {
  var textNode = new Text();
  textNode.textContent = v;
  this.childNodes = [textNode];
  return v
});

Element.prototype.addEventListener = function(t, l) {};

function escapeHTML(s) {
  return String(s)
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;');
}

function escapeAttribute(s) {
  return escapeHTML(s).replace(/"/g, '&quot;')
}

Element.prototype.nodeValue = null;

function Text(){}

Text.prototype.nodeType = 3;

Text.prototype.nodeName = '#text';

Text.prototype.__defineGetter__('textContent', function() {
  return escapeHTML(this.value || '');
});

Text.prototype.__defineSetter__('textContent', function(v) {
  this.value = v;
});

Text.prototype.__defineGetter__('nodeValue', function() {
  return escapeHTML(this.value || '');
});

Text.prototype.__defineSetter__('nodeValue', function(v) {
  this.value = v;
});

Text.prototype.__defineGetter__('length', function() {
  return (this.value || '').length;
});

Text.prototype.replaceData = function(offset, length, str) {
  this.value = this.value.slice(0, offset) + str + this.value.slice(offset + length);
};

function Comment(){}

Comment.prototype.nodeType = 8;

Comment.prototype.nodeName = '#comment';

Comment.prototype.__defineGetter__('data', function() {
  return this.value
});

Comment.prototype.__defineSetter__('data', function(v) {
  this.value = v;
});

Comment.prototype.__defineGetter__('outerHTML', function() {
  return '<!--' + escapeHTML(this.value || '') + '-->'
});

Comment.prototype.__defineGetter__('nodeValue', function() {
  return escapeHTML(this.value || '');
});

Comment.prototype.__defineSetter__('nodeValue', function(v) {
  this.value = v;
});

function defineParentNode (obj) {
  obj.__defineGetter__('parentNode', function () { return this.parentElement });
}

defineParentNode(Element.prototype);
defineParentNode(Comment.prototype);
defineParentNode(Text.prototype);
defineParentNode(Node.prototype);

var htmlElement = {
  Document: Document,
  Node: Node,
  Element: Element,
  Comment: Comment,
  Text: Text,
  document: new Document(),
};

var hyperscript = createCommonjsModule(function (module) {
var w = typeof window === 'undefined' ? htmlElement : window;
var document = w.document;
var Text = w.Text;

function context () {

  var cleanupFuncs = [];

  function h() {
    var args = [].slice.call(arguments), e = null;
    function item (l) {
      var r;
      function parseClass (string) {
        // Our minimal parser doesn’t understand escaping CSS special
        // characters like `#`. Don’t use them. More reading:
        // https://mathiasbynens.be/notes/css-escapes .

        var m = browserSplit(string, /([\.#]?[^\s#.]+)/);
        if(/^\.|#/.test(m[1]))
          e = document.createElement('div');
        forEach(m, function (v) {
          var s = v.substring(1,v.length);
          if(!v) return
          if(!e)
            e = document.createElement(v);
          else if (v[0] === '.')
            classList(e).add(s);
          else if (v[0] === '#')
            e.setAttribute('id', s);
        });
      }

      if(l == null)
        ;
      else if('string' === typeof l) {
        if(!e)
          parseClass(l);
        else
          e.appendChild(r = document.createTextNode(l));
      }
      else if('number' === typeof l
        || 'boolean' === typeof l
        || l instanceof Date
        || l instanceof RegExp ) {
          e.appendChild(r = document.createTextNode(l.toString()));
      }
      //there might be a better way to handle this...
      else if (isArray(l))
        forEach(l, item);
      else if(isNode(l))
        e.appendChild(r = l);
      else if(l instanceof Text)
        e.appendChild(r = l);
      else if ('object' === typeof l) {
        for (var k in l) {
          if('function' === typeof l[k]) {
            if(/^on\w+/.test(k)) {
              (function (k, l) { // capture k, l in the closure
                if (e.addEventListener){
                  e.addEventListener(k.substring(2), l[k], false);
                  cleanupFuncs.push(function(){
                    e.removeEventListener(k.substring(2), l[k], false);
                  });
                }else{
                  e.attachEvent(k, l[k]);
                  cleanupFuncs.push(function(){
                    e.detachEvent(k, l[k]);
                  });
                }
              })(k, l);
            } else {
              // observable
              e[k] = l[k]();
              cleanupFuncs.push(l[k](function (v) {
                e[k] = v;
              }));
            }
          }
          else if(k === 'style') {
            if('string' === typeof l[k]) {
              e.style.cssText = l[k];
            }else{
              for (var s in l[k]) (function(s, v) {
                if('function' === typeof v) {
                  // observable
                  e.style.setProperty(s, v());
                  cleanupFuncs.push(v(function (val) {
                    e.style.setProperty(s, val);
                  }));
                } else
                  var match = l[k][s].match(/(.*)\W+!important\W*$/);
                  if (match) {
                    e.style.setProperty(s, match[1], 'important');
                  } else {
                    e.style.setProperty(s, l[k][s]);
                  }
              })(s, l[k][s]);
            }
          } else if(k === 'attrs') {
            for (var v in l[k]) {
              e.setAttribute(v, l[k][v]);
            }
          }
          else if (k.substr(0, 5) === "data-") {
            e.setAttribute(k, l[k]);
          } else {
            e[k] = l[k];
          }
        }
      } else if ('function' === typeof l) {
        //assume it's an observable!
        var v = l();
        e.appendChild(r = isNode(v) ? v : document.createTextNode(v));

        cleanupFuncs.push(l(function (v) {
          if(isNode(v) && r.parentElement)
            r.parentElement.replaceChild(v, r), r = v;
          else
            r.textContent = v;
        }));
      }

      return r
    }
    while(args.length)
      item(args.shift());

    return e
  }

  h.cleanup = function () {
    for (var i = 0; i < cleanupFuncs.length; i++){
      cleanupFuncs[i]();
    }
    cleanupFuncs.length = 0;
  };

  return h
}

var h = module.exports = context();
h.context = context;

function isNode (el) {
  return el && el.nodeName && el.nodeType
}

function forEach (arr, fn) {
  if (arr.forEach) return arr.forEach(fn)
  for (var i = 0; i < arr.length; i++) fn(arr[i], i);
}

function isArray (arr) {
  return Object.prototype.toString.call(arr) == '[object Array]'
}
});

// const p = 'bindery-';
var p = '📖-';

var prefix = function prefix(str) {
  return '' + p + str;
};
var prefixClass = function prefixClass(str) {
  return '.' + prefix(str);
};

var c = function c(str) {
  if (str[0] === '.') {
    return prefixClass(str.substr(1));
  }
  return prefix(str);
};

___$insertStyle("@charset \"UTF-8\";@media screen{.📖-viewing .📖-controls{display:flex!important}}.📖-controls{font:16px/1.4 -apple-system,BlinkMacSystemFont,Roboto,sans-serif;display:none;flex-direction:row;align-items:start;position:fixed;top:0;left:0;right:0;z-index:1;margin:auto;color:#000;padding:10px;overflow:visible;-webkit-font-smoothing:antialiased}.📖-controls *{font:inherit;color:inherit;margin:0;padding:0;box-sizing:border-box}.📖-controls a{color:var(--bindery-ui-accent,#0000c5);text-decoration:none}.📖-row{flex-wrap:wrap;align-items:start;cursor:default;user-select:none}.📖-row,.📖-title{position:relative;display:flex}.📖-title{padding:8px 16px;transition:opacity .2s;white-space:nowrap;overflow:hidden;margin-right:auto}.📖-print-options{display:flex;opacity:0;max-width:0;border-radius:4px;background:#e4e4e4;overflow:hidden;transition:all .4s;flex-wrap:nowrap;transition-delay:.2s}[bindery-view-mode=print] .📖-print-options{max-width:720px;margin-right:12px;opacity:1}.📖-in-progress .📖-print-options{opacity:0;display:none}.📖-controls .📖-print-options .📖-control{background-color:transparent}.📖-spinner{border:1px solid transparent;border-left-color:#000;width:20px;height:20px;border-radius:50%;vertical-align:middle;opacity:0;pointer-events:none;transition:all .2s;margin-right:16px}.📖-in-progress .📖-spinner{opacity:1;animation:spin .6s linear infinite}.📖-debug .📖-spinner{animation:spin 2s linear infinite}.📖-spinner.📖-paused{animation:none}@keyframes spin{0%{transform:rotate(0)}to{transform:rotate(1turn)}}.📖-controls .📖-btn{-webkit-appearance:none;cursor:pointer;display:inline-block;margin-right:8px;text-decoration:none}.📖-controls .📖-btn:hover{background:#ddd}.📖-controls .📖-btn:active{background:#eee}.📖-controls .📖-btn:last-child{margin-right:0}.📖-controls .📖-control{border-radius:4px;color:#000;padding:8px 16px;background-color:#f4f4f4;border:0}.📖-controls .📖-btn-light{background:none;border:none}.📖-controls .📖-btn-main{background:var(--bindery-ui-accent,#0000c5);border-color:var(--bindery-ui-accent,#0000c5);color:#fff}.📖-controls .📖-btn-main:hover{background:var(--bindery-ui-accent,#0000c5);opacity:.7}.📖-controls .📖-btn-main:active{background:#000;opacity:1}.📖-view-row{margin-left:auto;margin-right:12px;transition:all .5s}.📖-in-progress .📖-view-row{opacity:0;pointer-events:none}.📖-debug .📖-view-row{display:none}.📖-btn-print{transition:all .5s}.📖-in-progress .📖-btn-print{opacity:0;pointer-events:none}.📖-debug .📖-btn-print{display:none}.📖-controls .📖-select-wrap{padding-right:32px;background-image:url(\"data:image/svg+xml;charset=utf-8,%3Csvg width='32' height='32' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath d='M11 14l5 5 5-5' stroke='%23000' fill='none'/%3E%3C/svg%3E\");background-repeat:no-repeat;background-position:100%;transition:all .2s;white-space:nowrap;width:100%}.📖-controls .📖-select-wrap:active,.📖-controls .📖-select-wrap:hover{background-color:#ddd}.📖-controls .📖-select-wrap.📖-hidden-select{max-width:0;padding-left:0;padding-right:0;border-left-width:0;border-right-width:0;color:transparent}.📖-select{position:absolute;top:0;left:0;opacity:0;-webkit-appearance:none;-moz-appearance:none;padding:8px 16px;color:#000;border:transparent;width:100%}.📖-debug-controls{display:none}.📖-debug .📖-in-progress .📖-debug-controls{display:block}.📖-refresh-btns{opacity:0;position:absolute;top:0;left:0;padding:8px 0;transition:all .2s}.📖-in-progress .📖-refresh-btns{display:none}.📖-refresh-btns a{margin-left:1em;cursor:pointer}.📖-refresh-btns a:hover{color:#000}@media screen and (max-width:720px){.📖-print-options{max-width:unset;max-height:0}[bindery-view-mode=print] .📖-print-options{max-width:unset;max-height:240px;margin:0}.📖-root{transition:all .2s}[bindery-view-mode=print].📖-root{padding-top:120px}.📖-controls{height:98px}.📖-view-row{margin-left:auto}.📖-print-options{top:54px;right:10px;position:fixed;margin:0}.📖-print-options .📖-row{margin:0}}@media screen and (max-width:500px){[bindery-view-mode=print].📖-root{padding-top:190px}[bindery-view-mode=print].📖-root .📖-controls{background:#f4f4f4;height:170px}.📖-print-options{flex-direction:column;align-items:stretch;left:10px}.📖-hidden-select{display:none}}");

var title = function title() {
  for (var _len = arguments.length, arg = Array(_len), _key = 0; _key < _len; _key++) {
    arg[_key] = arguments[_key];
  }

  return hyperscript.apply(undefined, [c('.title')].concat(arg));
};

var row = function row() {
  for (var _len3 = arguments.length, arg = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    arg[_key3] = arguments[_key3];
  }

  return hyperscript.apply(undefined, [c('.row')].concat(arg));
};

// Button
var btn = function btn() {
  for (var _len4 = arguments.length, arg = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    arg[_key4] = arguments[_key4];
  }

  return hyperscript.apply(undefined, ['button' + c('.control') + c('.btn')].concat(arg));
};

var btnMain = function btnMain() {
  for (var _len6 = arguments.length, arg = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
    arg[_key6] = arguments[_key6];
  }

  return hyperscript.apply(undefined, ['button' + c('.control') + c('.btn') + c('.btn-main')].concat(arg));
};

var select = function select() {
  var selectVal = hyperscript(c('.select-val'), 'Value');

  for (var _len7 = arguments.length, arg = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
    arg[_key7] = arguments[_key7];
  }

  var selectEl = hyperscript.apply(undefined, ['select.' + c('select')].concat(arg));
  var updateVal = function updateVal() {
    selectVal.textContent = selectEl.options[selectEl.selectedIndex].text;
  };
  selectEl.addEventListener('change', updateVal);
  updateVal();
  return hyperscript('' + c('.select-wrap') + c('.control'), selectVal, selectEl);
};

var option = function option() {
  for (var _len8 = arguments.length, arg = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
    arg[_key8] = arguments[_key8];
  }

  return hyperscript.apply(undefined, ['option'].concat(arg));
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};











































var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var supportsCustomPageSize = !!window.chrome && !!window.chrome.webstore;

var Controls = function Controls(availableOptions, initialState, actions) {
  classCallCheck(this, Controls);
  var Mode = availableOptions.Mode,
      Paper = availableOptions.Paper,
      Layout = availableOptions.Layout,
      Marks = availableOptions.Marks;


  var viewSelect = void 0;
  var marksSelect = void 0;
  var spinner = void 0;

  var print = function print() {
    actions.setMode(Mode.PRINT);

    var sel = viewSelect.querySelector('select');
    sel.value = Mode.PRINT;
    sel.dispatchEvent(new Event('change'));

    setTimeout(window.print, 10);
  };

  var printBtn = btnMain({ onclick: print }, 'Print');

  var paperSizes = (supportsCustomPageSize ? [option({ value: Paper.AUTO }, 'Auto'), option({ value: Paper.AUTO_BLEED }, 'Auto + Bleed'), option({ value: Paper.AUTO_MARKS }, 'Auto + Marks'), option({ value: Paper.LETTER_PORTRAIT }, 'Letter Portrait'), option({ value: Paper.LETTER_LANDSCAPE }, 'Letter Landscape'), option({ value: Paper.A4_PORTRAIT }, 'A4 Portrait'), option({ value: Paper.A4_LANDSCAPE }, 'A4 Landscape')] : [option({ value: Paper.LETTER_PORTRAIT, selected: true }, 'Default Page Size *'), option({ disabled: true }, 'Only Chrome supports custom page sizes. Set in your browser\'s print dialog instead.')]).map(function (opt) {
    if (opt.value === initialState.paper) {
      opt.selected = true;
    }
    return opt;
  });

  var updateSheetSizeNames = function updateSheetSizeNames() {
    if (!supportsCustomPageSize) return;
    var size = actions.getPageSize();
    var sizeName = size.width + ' \xD7 ' + size.height;
    paperSizes[0].textContent = '' + sizeName;
    paperSizes[1].textContent = sizeName + ' + Bleed';
    paperSizes[2].textContent = sizeName + ' + Marks';
  };
  updateSheetSizeNames();

  var updatePaper = function updatePaper(e) {
    var newVal = e.target.value;
    actions.setPaper(newVal);
    if (newVal === Paper.AUTO || newVal === Paper.AUTO_BLEED) {
      marksSelect.classList.add(c('hidden-select'));
    } else {
      marksSelect.classList.remove(c('hidden-select'));
    }
  };

  var sheetSizeSelect = select.apply(undefined, [{ onchange: updatePaper }].concat(toConsumableArray(paperSizes)));

  var layoutSelect = select.apply(undefined, [{ onchange: function onchange(e) {
      actions.setLayout(e.target.value);
      updateSheetSizeNames();
    } }].concat(toConsumableArray([option({ value: Layout.PAGES }, '1 Page / Sheet'), option({ value: Layout.SPREADS }, '1 Spread / Sheet'), option({ value: Layout.BOOKLET }, 'Booklet Sheets')].map(function (opt) {
    if (opt.value === initialState.layout) {
      opt.selected = true;
    }
    return opt;
  }))));
  var arrangement = row(layoutSelect);

  marksSelect = select.apply(undefined, [{ onchange: function onchange(e) {
      return actions.setMarks(e.target.value);
    } }].concat(toConsumableArray([option({ value: Marks.NONE }, 'No Marks'), option({ value: Marks.CROP, selected: true }, 'Crop Marks'), option({ value: Marks.BLEED }, 'Bleed Marks'), option({ value: Marks.CROP }, 'Crop and Bleed')].map(function (opt) {
    if (opt.value === initialState.marks) {
      opt.selected = true;
    }
    return opt;
  }))));
  if (supportsCustomPageSize) {
    marksSelect.classList.add(c('hidden-select'));
  }
  var marks = row(marksSelect);
  var sheetSize = row(sheetSizeSelect);

  var headerContent = hyperscript('span', 'Loading');

  var playSlow = void 0;
  var step = btn('→', {
    style: { display: 'none' },
    onclick: function onclick() {
      window.binderyDebug.step();
      document.scrollingElement.scrollTop = document.scrollingElement.scrollHeight;
    }
  });
  var pause = btn('❙❙', {
    onclick: function onclick() {
      window.binderyDebug.pause();
      spinner.classList.add(c('paused'));
      pause.style.display = 'none';
      playSlow.style.display = '';
      step.style.display = '';
    }
  });
  playSlow = btn('▶️', {
    style: { display: 'none' },
    onclick: function onclick() {
      window.binderyDebug.resume();
      spinner.classList.remove(c('paused'));
      playSlow.style.display = 'none';
      pause.style.display = '';
      step.style.display = 'none';
    }
  });
  var debugDone = btn('Finish', {
    onclick: function onclick() {
      spinner.classList.remove(c('paused'));
      window.binderyDebug.finish();
    }
  });

  var debugControls = hyperscript(c('.debug-controls'), pause, playSlow, step, debugDone);

  spinner = hyperscript(c('.spinner'));
  var header = title(headerContent);

  this.setInProgress = function () {
    headerContent.textContent = 'Paginating';
  };

  var lastUpdate = 0;
  this.updateProgress = function (count) {
    var t = performance.now();
    if (t - lastUpdate > 100) {
      lastUpdate = t;
      headerContent.textContent = count + ' Pages';
    }
  };

  this.setDone = function (length) {
    headerContent.textContent = length + ' Pages';
  };

  this.setInvalid = function () {};

  printBtn.classList.add(c('btn-print'));
  var options = row(arrangement, sheetSize, marks);
  options.classList.add(c('print-options'));

  viewSelect = select.apply(undefined, [{ onchange: function onchange(e) {
      return actions.setMode(e.target.value);
    } }].concat(toConsumableArray([option({ value: Mode.PREVIEW }, 'Preview'), option({ value: Mode.FLIPBOOK }, 'Flipbook'), option({ value: Mode.PRINT }, 'Print Preview')].map(function (opt) {
    if (opt.value === initialState.mode) {
      opt.selected = true;
    }
    return opt;
  }))));
  var viewRow = row(viewSelect);
  viewRow.classList.add(c('view-row'));

  this.element = hyperscript(c('.controls'), header, debugControls, viewRow, options, printBtn);
};

return Controls;

})));
//# sourceMappingURL=bindery-controls.umd.js.map
